# coding: utf-8
"""
Модуль функций подсчета показателей.
"""

from django.db.models import Q

from kinder.core.declaration_status.models import DSS


from kinder.core.privilege.models import Privilege
from kinder.core.dict.models import HealthNeed
from kinder.core.alloc.helpers import GroupHelper

from constants import (AGE_SUBCATEGORY_MAP, AGE_CATEGORY_MAP,
                       GOVERMENT, get_last_day_of_academic_year)
from gisdo.alchemy_queries import BaseIndex

# статусы заявлений для запросов
APPROVED_STATUS_CODES = [DSS.ZAGS_CHECKING, DSS.DUL_CONFIRMATING,
                         DSS.PRIV_CONFIRMATING, DSS.MED_CONFIRMATING,
                         DSS.TUTOR_CONFIRMATING, DSS.WANT_CHANGE_DOU,
                         DSS.REGISTERED]
REFUSED_STATUS_CODES = [DSS.REFUSED]
EXCLUDE_STATUSES = [DSS.ACCEPTED, DSS.ARCHIVE]

FEDERAL = 1
MUNICIPAL = 2
REGIONAL = 3
PRIVELEGE_TYPE = {
    FEDERAL: u'Федеральная',
    MUNICIPAL: u'Муниципальная',
    REGIONAL: u'Региональная'
}


def _get_base_queue_for_unit(calc, unit_id, dou_type, ages_const):

    result = []
    if unit_id in calc.queue_cache and dou_type in calc.queue_cache[unit_id]:
        result = calc.queue_cache[unit_id][dou_type][ages_const]

    return result


def count_of_children_in_short_stay(calc, unit_id, ages_const, dou_type=GOVERMENT):
    """
    Количество детей в очереди в группы кратковременного пребывания
    Считаем как заявки с типом work_type = кратковременный (3-5)(id=4)
    @param calc: экземпляр класса расчета
    @param ages_const: Константа границ возрастной категории
    @param unit_id: id учреждения, для которого считается очередь
    @return:
    """
    qs = _get_base_queue_for_unit(calc, unit_id, dou_type, ages_const)
    qs = filter(
        lambda x: x['status__code'] in [DSS.REGISTERED, DSS.PRIV_CONFIRMATING]
        and (not x['desired_date']
             or x['desired_date'] <= get_last_day_of_academic_year())
        and x['work_type__code'] == '3-5', qs)
    return len(qs)


def count_of_children_in_queue_base(calc, unit_id, ages_const, dou_type=GOVERMENT):
    """
    Очередь как есть
    @param calc: экземпляр класса расчета
    @param ages_const: Константа границ возрастной категории
    @param unit_id: id учреждения, для которого считается очередь
    """
    qs = _get_base_queue_for_unit(calc, unit_id, dou_type, ages_const)
    return len(qs)


def count_of_children_in_privilege_queue(calc, unit_id, ages_const, privilege_type=None, dou_type=GOVERMENT):
    """
    Очередь только по льготникам
    отличается по списку статусов и наличию лгот
    @param calc: экземпляр класса расчета
    @param ages_const: Константа границ возрастной категории
    @param unit_id: id учреждения, для которого считается очередь
    @return:
    """
    qs = _get_base_queue_for_unit(calc, unit_id, dou_type, ages_const)
    qs = filter(lambda x: x['best_privilege__id'], qs)
    qs = filter(lambda x: x['status__code'] == DSS.REGISTERED, qs)
    qs = filter(lambda x: not x['desired_date'] or x['desired_date'] <= get_last_day_of_academic_year(), qs)
    if privilege_type:
        if privilege_type == REGIONAL:
            privilege_filter = Q(type=REGIONAL) | Q(type=MUNICIPAL)
        else:
            privilege_filter = Q(type=privilege_type)
        qs_pr = Privilege.objects.filter(privilege_filter).values_list('id', flat=True)
        qs = filter(lambda x: x['best_privilege__id'] in qs_pr, qs)
    return len(qs)


def children_without_place_count(calc, unit_id, ages_const, dou_type=GOVERMENT):
    """
    Подсчет количества детей, не обеспеченных местом в ДОУ
    @param calc: экземпляр класса расчета
    @param ages_const: Константа границ возрастной категории
    @param unit_id: id учреждения, для которого считается очередь
    """
    base_qs = _get_base_queue_for_unit(calc, unit_id, dou_type, ages_const)
    base_qs = filter(lambda x: not x['desired_date'] or x['desired_date'] <= get_last_day_of_academic_year(), base_qs)
    return len(base_qs)


def children_with_health_need_count(calc, unit_id, ages_const, comp=0, dou_type=GOVERMENT):
    """
    Подсчет количества детей  в очереди, имеющих потребности по здоровью
    @param calc: экземпляр класса расчета
    @param ages_const: Константа границ возрастной категории
    @param unit_id: id учреждения, для которого считается очередь
    @param comp: 0 все группы
    comp 1 компенсирующие группы
    comp 2 оздоровительные группы
    """
    qs = _get_base_queue_for_unit(calc, unit_id, dou_type, ages_const)
    # При подсчете детей в очереди с потребностью по здоровью учитывать только актуальный спрос.
    qs = filter(lambda x: not x['desired_date'] or x['desired_date'] <= get_last_day_of_academic_year(), qs)
    if comp == 0:
        qs_hn = HealthNeed.objects.exclude(name=u'Нет').values_list('id', flat=True)
    elif comp == 1:
        #id потребностей из документа расчета показателей
        qs_hn = range(1, 10) + [12, 13]
    elif comp == 2:
        qs_hn = [14, 15, 16]
    qs = filter(lambda x: x['children__health_need_id'] in qs_hn, qs)
    return len(qs)


def postponed_queue_children_count(calc, unit_id, ages_const, dou_type=GOVERMENT):
    """
    Количетство детей в отложенной очереди
    Заявки с желаемой датой зачисления больше 10.09.текущий_учебный_год
    + если комплектование прошло (дата окончания < текущей) то те дети у кого дата
    желаемого зачисления пустая либо меньше 31.12.текущий_год
    @param calc: экземпляр класса расчета
    @param ages_const: Константа границ возрастной категории
    @param unit_id: id учреждения, для которого считается очередь
    """
    base_qs = _get_base_queue_for_unit(calc, unit_id, dou_type, ages_const)

    base_qs = filter(lambda x: x['desired_date'] and x['desired_date'] > get_last_day_of_academic_year(), base_qs)
    count = len(base_qs)

    return count


#----------------------------------------------
# расчеты по количеству свободных мест
#----------------------------------------------
#----------------------------------------------
# расчеты по количеству свободных мест
#----------------------------------------------
class CommonCountAlgorithm(object):

    def get_result_set(self, cache):

        return cache


class ShortStayAlgorithm(object):

    def get_result_set(self, cache):

        result_set = []

        for group in cache:
            if group.work_type and group.work_type.code == '3-5':
                result_set.append(group)

        return result_set


class HealthNeedIndex(object):

    def get_result_set(self, cache):

        result_set = []

        for group in cache:
            # (#85387) Считать, как количество всех детей зачисленных в группы типа "Оздоровительные"
            # или "Компенсирующие" без учета наличия или отсутствия потребности по здоровью.
            if group.type and group.type.code in ['health', 'compensative']:
                result_set.append(group)

        return result_set


class CapacitiesIndex(BaseIndex):
    """
    """

    COMMON_COUNT = 1
    SHORT_STAY = 2
    HEALTH_NEED = 3

    ALGORITHM_MAP = {
        COMMON_COUNT: CommonCountAlgorithm,
        SHORT_STAY: ShortStayAlgorithm,
        HEALTH_NEED: HealthNeedIndex
    }

    class Ctx(object):

        kind = 1

        def __init__(self, unit_id):

            self.group_unit = unit_id

    def __init__(self, unit):

        super(CapacitiesIndex, self).__init__(unit, session=None)

        self._cache = GroupHelper(CapacitiesIndex.Ctx(self.unit.id)).getList()

    def get_count(self, index_type=COMMON_COUNT, dou_type=GOVERMENT, **kwargs):

        if (not self.dou_type and dou_type != GOVERMENT) or (self.dou_type and self.dou_type.code != dou_type):
            return 0

        result_list = CapacitiesIndex.get_algorithm(index_type)().get_result_set(cache=self._cache)

        age_category = kwargs.get('age_category', None)
        total = 0
        for group in result_list:

            # Если у группы есть возрастная подкатегория, то соотношение проводится по ней
            if group.sub_age_cat:
                if group.sub_age_cat.code in AGE_SUBCATEGORY_MAP[age_category]:
                    total += group.cntFree
            # в противном случае по возрастной категории, которая обязательна.
            else:
                if group.age_cat.code in AGE_CATEGORY_MAP[age_category]:
                    total += group.cntFree
        return total